    {% for maintainer in product.maintainers %}
      <a href="mailto:{{ maintainer.email }}">{{ maintainer.name }}</a>{% if not loop.last %},<br/>{% endif %}
    {% else %}
      <a href="mailto:kde-devel@kde.org">The KDE Community</a>
    {% endfor %}
