{% extends "base.html" %}

{% block page_title %}{{title}}{% endblock %}

{% block head %}
<script type="text/javascript" src="{{resources}}/notetip.js"></script>
<script type="text/javascript" src="{{resources}}/frameworks.js"></script>
<link rel="stylesheet" type="text/css" href="{{resources}}/frameworks.css" />
<script>
    $(document).ready(main);
</script>
{% endblock %}

{% block title %}
KDE Products
{% endblock %}

{% block content %}
<div class="textblock">
  {% for para in intro %}
    <p>{{ para }}</p>
  {% endfor %}
</div>

<section class="products px-0">
  {% for product in product_list %}
    <div class="product my-3 ae-card p-4">
      <h2 class="h2 mt-0 mb-3 d-flex">
        <a name="prod-{{ product.name }}" href="{{ product.href }}">{{ product.fancyname }}</a>
        {% if qch or man %}
          <p class="prod-download ml-auto h6">
            {% if qch %} [<a href="qch/{{product.name}}.qch">QCH file</a>]{% endif %}
            {% if man %} <a href="qch/{{product.name}}.man">MAN</a>{% endif %}
          </p>
        {% endif %}
      </h2>
      <div class="d-flex">
        <img src="{% if product.logo_url %} {{ product.logo_url }} {% else %}{{resources}}/default_product.png{% endif %}"
             style="min-width:80px; width: 80px;" class="align-self-start my-0 mr-4" />
        <div>
          <p>{{ product.description }}</p>
          <div>
            <b>Maintainer{% if product.maintainers | length > 1 %}s{% endif %}:</b>
            {% for maintainer in product.maintainers %}
              <a href="mailto:{{ maintainer.email }}">{{ maintainer.name }}</a>
              {% if not loop.last %},{% endif %}
            {% else %}
              <a href="mailto:kde-devel@kde.org">The KDE Community</a>
            {% endfor %}
          </div>
          {% if product.platforms %}
            <p>
              <b>Platform{% if product.platforms | length > 1 %}s{% endif %}:</b> {{ product.platforms | sort | join(' | ') }}
            </p>
          {% endif %}
        </div>
      </div>
    </div>
  {% endfor %}
</section>
{% endblock content %}

{% block sidebar %}
  {% include "frontpage-sidebar.html" %}
{% endblock %}
