{% extends "base.html" %}

{% block page_title %}{{ title }}{% endblock %}

{% block content %}

{{ docs }}

{% endblock %}

{% block sidebar %}
    {% include "qdoc-sidebar.html" %}
{% endblock %} {# sidebar #}
