{% if class_map %}
  <h2 class="h4">Class Picker</h2>
  <form name="guideform" id="class-picker">
    <select class="form-control" name="guidelinks" onChange="window.location=document.guideform.guidelinks.options[document.guideform.guidelinks.selectedIndex].value">
      <option value="annotated.html">-- Choose --</option>
      {% for class in class_map.classes %}
      <option value="{{class.filename}}">{{class.classname}}</option>
      {% endfor %}
    </select>
  </form>
{% endif %}

<h2 class="h4">Quick Links</h2>
<a href="#cp-menu" class="sr-only">Skip menu</a>
<div class="menu-content">
{% if doxygen_menu %}
  <div class="list-group">
  {% for entry in doxygen_menu.entries %}
    <a class="list-group-item list-group-item-action" href="{{entry.href}}">{{entry.text}}</a>
  {% endfor %}
  </div>
{% endif %}
</div>

{% if group_menu %}
  <a name="cp-menu" />
  <h2 class="h4">{{group_menu.group_title}}</h2>
  <a href="#cp-skip-project" class="sr-only">Skip menu "{{group_menu.group_title}}"</a>
  <ul>
    {% for section in group_menu.sections %}
      <li>{{section.title}}
        <ul>
          {% for member in section.members %}
            <li><a href="{{member.href}}">{{member.name}}</a></li>
          {% endfor %}
        </ul>
      </li>
    {% endfor %}
  </ul>
{% endif %}
