/**
 * SPDX-FileCopyrightText: (c) 2016 Olivier Churlaud <olivier@churlaud.com>
 * SPDX-FileCopyrightText: (c) 2020 Carson Black <uhhadd@gmail.com>
 * SPDX-FileCopyrightText: (c) 2020 Carl Schwan <carl@carlschwan.eu>
 * 
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */
:root {
  --base: white;
  --on-base: black;
  --plasma-blue: #54a3d8;
  --on-plasma-blue: white;
  --card: white;
  --on-card: black;
  --borders: rgba(0,0,0,0.3);
  --borders-faint: rgba(0,0,0,0.1);
  --list-header-bg: #eff0f1;
}

@media (prefers-color-scheme: dark) {
  table tr:nth-child(odd) {
    background-color: transparent !important;
  }
  .mdescLeft, .mdescRight, .memTemplItemLeft, .memTemplItemRight, .memTemplParams {
    background-color: transparent !important;
  }
  :root {
    --base: #232629;
    --on-base: #eff0f1;
    --card: rgba(255,255,255,0.02);
    --on-card: #eff0f1;
    --borders: rgba(255,255,255,0.2);
    --borders-faint: rgba(255,255,255,0.05);
    --plasma-blue: #98c8e8;
    --on-plasma-blue: black;
    --list-header-bg: #31363b;

    background-color: var(--base);
  }
  .blockquote {
    background-color: var(--card);
  }
}
.memItemLeft, .memItemRight {
  background-color: var(--base);
}
.memtitle {
 display:none
}
.memdoc {
 border:1px solid rgba(0, 0, 0, 0.4);
 border-radius:0 0 3px 3px;
}
.memproto,
dl.reflist dt {
 background:var(--plasma-blue);
 color:var(--on-plasma-blue);
 border-radius:0;
 border-top-left-radius: 4px;
 border-top-right-radius: 4px;
 text-shadow:none
}
.memproto a.el,
dl.reflist dt a.el {
 color:beige
}

.memname {
 color:var(--on-plasma-blue);
 text-shadow:none;
 word-break: break-word;
}
@media (prefers-color-scheme:dark) {
 table {
  background-color:transparent!important;
  color:var(--on-base)!important
 }
 .memproto,
 dl.reflist dt {
  background:var(--card)!important;
  color:var(--on-card)!important
 }
 .memname {
  color:var(--on-card)
 }
 .memproto table tr,
 .mlabels tr,
 table.memberdecls tr.heading,
 table.mlabels tr,
 table.params tr {
  background-color:transparent!important;
  color:var(--on-base)!important
 }
 div.memproto {
  border:0
 }
 span.stringliteral {
    color: #0040ff;
 }
 .comment {
   color: #ff0202;
 }
 .keyword {
   color: #00ff00;
 }
 span.lineno, span.lineno a {
   background-color: transparent;
 }
 span.comment {
   color: #fdaaaa;
 }
 a.code, a.code:visited, a.line, a.line:visited {
   color: #6a9cff;
 }
 span.keywordtype {
   color: #ffa143;
 }
 .dyncontent img {
   filter: invert(0.85);
 }
 .classindex dl.odd {
   background-color: var(--base);
 }
}
.dyncontent img {
  filter: invert(0.01);
}
.memdoc,
dl.refllist dd {
 background:var(--card);
 color:var(--on-card)
}
.memSeparator {
 border-color:var(--borders)
}
@media (max-width: 900px) {
  .td-toc {
    height: auto;
  }
}
.mdescLeft,
.mdescRight,
.memItemLeft,
.memItemRight,
.memTemplItemLeft,
.memTemplItemRight,
.memTemplParams {
 background:var(--card);
 color:var(--on-card);
 border-color:var(--borders)
}
.memproto a.el,
.memproto a.elRef {
 color:#cce;
 font-weight:700
}
.memproto .paramtype {
 font-weight:700
}
.memproto .paramname {
 color:#f7800a;
 font-weight:700
}
.paramname {
  padding-right: 6px;
}
.memproto span.mlabel {
 padding:6px;
 border:0;
 border-radius:4px;
 background:var(--base);
 color:var(--on-base)
}

/* Use div.fragment, not pre.fragment. pre.fragment is used for @verbatim blocks, which must keep their border */
div.fragment {
  overflow-x: auto;
  background-color: var(--card);
  color: var(--on-card);
  border: 1px solid var(--borders);
  border-radius: 4px;
  padding: 10px;
  margin-top: 10px;
  margin-bottom: 10px;
}

pre.fragment {
  overflow-x: auto;
  background-color: var(--card);
  color: var(--on-card);
  border-color: var(--borders);
  white-space: pre;
  scrollbar-width: 0;
}
pre.fragment::-webkit-scrollbar {
  width: 0;
  height: 0;
}

.td-toc a {
  display: inline;
}

/**
 * Subgroup
 */

table.libraries {
  width: 100%;
  border-collapse: collapse;
  margin-bottom: 12px;
}

table.libraries td, table.libraries th {
  border: solid 1px var(--borders);
}

table.libraries h3 {
  margin: 0;
  padding: 0;
}

table.libraries td {
  padding: 0.5em;
}

table.libraries th {
  background: var(--plasma-blue);
  color: var(--on-plasma-blue);
  font-size: 1.1em;
  line-height: 1.7em;
  text-align: center;
}

@media (prefers-color-scheme: dark) {
  table.libraries th {
    background: var(--card);
    color: var(--on-card);
  }
}

.library-main-column {
  vertical-align: top;
}

.library-detail-column {
  text-align: center;
  width: 120px;
}

.library-fancyname {
  font-size: 120%;
}

.library-platforms {
  float: right;
  font-size: 80%;
}

.library-platform {
  color: #666;
  padding-left: 6px;
  margin-left: 6px;
  border-left: 1px solid #ccc;
}

.library-platform:first-child {
  padding-left: 0;
  border-left: none;
}

.library-platform a {
  text-decoration: none;
}

.library-platform-unsupported .library-platform-text {
  text-decoration: line-through;
  color: #aaa;
}

/* Platform filtering --------------------------------*/
.not-available-indicator, .available-indicator {
  display: none;
  float: left;
  padding-right: 0.5em;
}

.not-available .not-available-indicator {
  display: inline;
}

.available .available-indicator {
  display: inline;
}

.not-available {
  background-color: rgba(233, 61, 87, 0.2);
}

.not-available .library-description {
  display: none;
}

.library-platform-required .library-platform-text {
  color: green;
}

.library-platform-required.library-platform-unsupported .library-platform-text {
  color: red;
}

.warning-tooltip {
  width: 15px;
}

.fieldtable th {
  background: var(--plasma-blue);
  border-radius: 0;
  color: var(--on-plasma-blue);
}

/* Platform note -------------------------------------*/
.note-tip {
  display: none;
  z-index: 3000;
  position: absolute;

  background-color: #222;
  color: white;
  padding: 6px;
  border-radius: 5px;
}

a.note-close, a.note-close:active, a.note-close:visited {
  color: white;
  text-decoration: none;
  margin-left: 6px;
}

html, body, table, div, p, dl {
  font: 400 5.625vw/20px Noto Sans,-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Helvetica Neue,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
}
@media (min-width:320px) {
 html, body, table, div, p, dl {
  font: 400 17px/20px Noto Sans,-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Helvetica Neue,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
 }
}
@media (min-width:900px) {
 html, body, table, div, p, dl {
  font: 400 16px/20px Noto Sans,-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Helvetica Neue,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
 }
}
@media print {
 html, body, table, div, p, dl {
  font: 400 14px/20px Noto Sans,-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Helvetica Neue,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
 }
}

.memItemLeft {
  display: flex;
  justify-content: end;
}
.memItemRight {
  vertical-align: middle;
}
