{% extends "base.html" %}
{% block page_title %}{{ dox.projectname }} - {{ dox.title }}{% endblock %}

{% block title %}
{% if fullname %}
{{ fullname }}
{% else %}
{{ dox.projectname }}
{% endif %}
{% endblock title %}

{% block content %}

{% if classname %}
<p><a href="https://lxr.kde.org/ident?v=kf5-qt5&amp;_i={{ classname }}">Search for usage in LXR</a></p>
{% endif %}

{{ dox.content }}

<div class="copyrights mt-5">
  This file is part of the KDE documentation.<br/>
  Documentation copyright &copy; {{copyright}}.<br/>
  Generated on {{ dox.datetime }} by
  <a href="https://www.doxygen.org/">doxygen</a> {{ dox.doxygenversion }} written
  by <a href="mailto:dimitri@stack.nl">Dimitri van Heesch</a>, &copy;&nbsp;1997-2006
  <p>
  KDE's <a href="https://community.kde.org/Frameworks/Frameworks_Documentation_Policy">Doxygen guidelines</a> are available online.
  </p>
</div>

{% endblock %}

{% block sidebar %}
    {% include "library-sidebar.html" %}
{% endblock %} {# sidebar #}
