table.frameworks {
    width: 100%;
    border-collapse: collapse;
}
table.frameworks td, table.frameworks th{
    border: solid 1px #0057ae;
}

table.frameworks h3 {
    margin: 0;
    padding: 0;
}

table.frameworks td {
    padding: 0.5em;
}

table.frameworks th {
    background: #0057ae;
    color:white;
    font-size: 1.1em;
    line-height: 1.7em;
}

.framework-main-column {
    vertical-align: top;
}

.framework-detail-column {
    text-align: center;
    width: 120px;
}

.framework-fancyname {
    font-size: 120%;
}

.framework-platforms {
    float: right;
    font-size: 80%;
}

.framework-platform {
    color: #666;
    padding-left: 6px;
    margin-left: 6px;
    border-left: 1px solid #ccc;
}

.framework-platform:first-child {
    padding-left: 0;
    border-left: none;
}

.framework-platform a {
    text-decoration: none;
}

.framework-platform-unsupported .framework-platform-text {
    text-decoration: line-through;
    color: #aaa;
}

/* Platform filtering --------------------------------*/
.not-available-indicator, .available-indicator {
    display: none;
    float: left;
    padding-right: 0.5em;
}

.not-available .not-available-indicator {
    display: inline;
}

.available .available-indicator {
    display: inline;
}

.not-available {
    background-color: #f8f8f8;
}

.not-available .framework-description {
    display: none;
}

.framework-platform-required .framework-platform-text {
    color: green;
}

.framework-platform-required.framework-platform-unsupported .framework-platform-text {
    color: red;
}

/* Platform note -------------------------------------*/
.note-tip {
    display: none;
    z-index: 3000;
    position: absolute;

    background-color: #222;
    color: white;
    padding: 6px;
    border-radius: 5px;
}

a.note-close, a.note-close:active, a.note-close:visited {
    color: white;
    text-decoration: none;
    margin-left: 6px;
}
