<p>If you are looking for documentation about KDE software or how to use Plasma, please refer to our <a href="https://userbase.kde.org">userbase website</a>.</p>
<h2 class="h4">Other KDE Products</h2>
<ul>
  <li><a href="https://api.kde.org/ecm/">Extra CMake Modules</a></li>
</ul>
<h2 class="h4">Deprecated Products</h2>
<ul>
  <li><a href="https://api.kde.org/legacy/">Old KDE Versions</a></li>
<ul>
