/*
** HTML elements
*/
body {
    background:  url("https://cdn.kde.org/img/bg.png") repeat;
    margin: 0px;
}

body, table, div, p, dl {
    font-family: "Noto Sans",Verdana, Geneva, Arial, sans-serif;
    font-size: 13px;
    line-height: 1.3;
}

/*
** HTML Tags
*/

h1, h2, h3, h4 {
    color: #f7800a;
    background: transparent;
}

h1 {
    font-size: 1.7em;
}

h2 {
    font-size: 1.5em;
}

h3 {
    font-size: 1.4em;
}

h4 {
    font-weight: bold;
}

a:link {
    color: #0057ae;
    text-decoration: none;

}

a:hover {
    text-decoration: underline;
}

a:visited {
    color: #644A9B;
}

input, textarea, select {
    margin: 0.2em;
    padding: 0.1em;
    border: 1px solid;
}

.products {
    border-collapse:separate;
    border-spacing: 0 10px;
}
.product {
    background: #fefefe;
    margin-bottom: 40px;
}

.product-logo {
    padding: 20px;
}

.product-descr {
    padding: 15px;
    max-width: 500px
}

.product-name {
    font-weight: bold;
}

ul.list-unstyled {
    list-style-type: none;
    padding: 0;
}

ul.list-unstyled li {
    margin-top: 0.4em;
}

ul.list-unstyled li:first-child {
    margin-top:0;
}

.prod-platforms:before {
    content: "Platforms: ";
    font-weight: bold;
}

.prod-maintainers:before {
    content: "Maintainers: ";
    font-weight: bold;
}

.menu_box dl {
    padding: 0 0 0 10px;
}

.menu_box dt a {
    font-weight: bold;
}

.menu_box dt {
    font-weight: bold;
    margin-top: 0.8em;
}
.menu_box dt:first-child {
    margin-top: 0;
}
.menu_box dd {
    margin-left: 1em;
    color: #323232;
}

form {
    margin: 0;
    padding: 0;
}

/*
** Header
*/
#header {
    width: 100%;
    color: #535353;
}

#header_top {
    margin: 0 auto;
    padding: 0;
    width: 100%;
    vertical-align: middle;
    color: #ffffff;
    background-color: #0057ae;
}

#header_top div {
    margin: auto;
    padding: 0;
}

#header_top div div {
    margin: 0 auto;
    padding: 0;
    vertical-align: middle;
    max-width:1200px;
    font-size: 1.6em;
    font-weight: bold;
}

#header_top div div img {
    margin:4px 0px 6px 18px;
    vertical-align: middle;
}

#header_bottom {
    margin: 0 auto;
    padding: 0.1em 0em 0.3em 0;
    width: 95%;
    vertical-align: middle;
    text-align: right;
    max-width: 1200px;
}

#location {
    padding: 0 0 0 1.5em;
    line-height: normal;
    font-size: 1.1em;
    float: left;
}

#location ul {
    display: inline;
    margin: 0;
    padding: 0;
    list-style: none;
}

#location ul li {
    display: inline;
    white-space : nowrap;
    margin: 0;
    padding: 0 1em 0 0;
}

#menu {
    margin: 0 1.5em 0 0;
    text-align: right;
    line-height: normal;
    font-size: 1.1em;
    font-weight: bold;
}

#menu ul {
    display: inline;
    list-style: none;
    margin: 0;
    padding: 0;
    text-align: right;
}

#menu ul li {
    display: inline;
    white-space : nowrap;
    margin: 0;
    padding: 0 0 0 1em;
    text-align: right;
}

#menu ul li a {
    font-weight: bold;
}


/*
** Content
*/

#main {
    /* padding in px not ex because IE messes up 100% width tables otherwise */
    padding-left: 10px;
}

/* The project name (redundant as it is shown in the menu, too) */
div.clearer+h2 {
    display: none;
}

/* The Doxygen-provided page title, that is already displayed in the header */
div.header, div#top {
    display: none;
}

/* The Doxygen content feels "out of place" if it has a margin different from the rest of the page */
div.contents {
    margin-left: 0;
    margin-right: 0;
}

/* Provides the thin vertical grey line on the left of the menu */
#body_wrapper {
    margin: 0 auto;
    width: 95%;
    max-width: 1200px;
}

#right {
    float: right;
    width: 73%;
}

/* Additional info at the top of classes */
.additional_info {
    float: right;
    text-align: right;
    list-style-type: none;
    border: 1px solid #0057ae;
    padding: 0px 20px;
}
.additional_info li {
    padding: 3px 0px;
}

/* Comments */
#comments {
    border-top: 4px double #0057ae;
    margin-top: 20px;
    padding-top: 20px;
}

.juvia-comment .juvia-metadata {
    border: 1px solid #A8B8D9;
    color: #253555;
    font-weight: bold;
    text-shadow: 0px 1px 1px rgba(255, 255, 255, 0.9);
    background: url('karchive/html/nav_f.png') repeat-x #E2E8F2 !important;
    box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.15);
    border-radius: 4px;
}

/*
** Listing
*/
span.lineno {
    border-right: 1px solid #ddd;
    background-color: transparent;
    color: #666;
}

span.lineno a {
    background-color: transparent;
}

/* Use div.fragment, not pre.fragment. pre.fragment is used for @verbatim blocks, which must keep their border */
div.fragment {
    background-color: #fefefe;
    border: none;
    border-radius: 5px;
}

/*
** Left Menu
*/

#left {
    float: left;
    margin: 0;
    padding: 0;
    width: 25%;
    max-width: 300px;
}

.menu_box {
    padding: 0.7em 0 0 0;
}

.menu_box p {
    margin-left: 1em;
    margin-right: 1em;
}

.menu_box li {
    list-style-type: none;
    margin-left: /*0.1em*/ -1.5em;
}

.menu_box ul ul {
    padding-left: 0;
}

.menu_box li li {
    margin-left: 1em;
}

.menutitle {
    margin: 0.6em 0 1.2em 0;
    padding:0;
    color: #ffffff;
    background-color: #0057ae;
}

.menutitle div h2 {
    margin: 0;
    padding: 0.2em 0 0.3em 1.3em;
    line-height:1.2em;
    font-size: 120%;
    font-weight: normal;
    color: #ffffff;
}

.clearer {
    clear: both;
    height: 1px;
}


/*
** Footer
*/
.copyrights {
    border-top: 1px solid #dddddd;
    margin-top: 10px;
    padding-top: 5px;
    text-align: center;
    color: #555555;
}

#footer {
    color: #eeeeee;
    background: #0057ae;
    width: 100%;
    margin:auto;
    margin-top: 20px;
}

#footer_text {
    margin: 0 auto;
    padding: 1em 0 1em 3.5em;
    width: 51.5em;
}

#footer a:link, #footer a:visited {
    color: #fff;
    font-weight: bold;
}


/**
 * Fragments
 */
.fragment .line {
    line-height: 1.4;
}

/**
 * classes
 */
.memproto, dl.reflist dt {
    background: #0057ae;
    border-radius: 0;
    border: 1px solid #0057ae;
}

.memname {
    color: #eeeeee;
    text-shadow: none;
}

.memproto a.elRef{
    color: #aaaabb;
    font-weight: bold;
}
.memproto a.el {
    color: #ccccee;
    font-weight: bold;
}

.memproto .paramname {
    color: #f7800a;
}

.memproto span.mlabel {
    background: #bbbbff;
    color: #005766;
}
