<p class="text-white">Report problems with this website to <a href="https://invent.kde.org/frameworks/kapidox/-/issues">our bug tracking system</a>.<br />
Contact the specific authors with questions and comments about the page contents.<p>
<p class="text-white" >KDE<sup>&#174;</sup> and <a href="/media/images/kde_gear_black.png">the K Desktop Environment<sup>&#174;</sup> logo</a> are registered trademarks of <a href="https://ev.kde.org/" title="Homepage of the KDE non-profit Organization">KDE e.V.</a> | <a href="https://kde.org/community/whatiskde/impressum">Legal</a></p>
